@@ -154,7 +154,7 @@ void init_keyevents(void) {
 
 	home = getenv("HOME");
 	if (!home)
-		eprintf("No HOME in environment\n");
+		eprintf("No HOME in environment");
 
 	confhome = getenv("XDG_CONFIG_HOME");
 
@@ -345,10 +345,17 @@ void feh_event_invoke_action(winwidget winwid, unsigned char action)
 				feh_reload_image(winwid, 1, 1);
 			else
 				winwidget_destroy(winwid);
-		} else if (winwid->type == WIN_TYPE_THUMBNAIL)
-			fputs("actions from the main thumb window aren't currently supported!\n"
-					"For now, open the image to perform the action on it.\n",
-					stdout);
+		} else if (winwid->type == WIN_TYPE_THUMBNAIL) {
+			feh_file *thumbfile;
+			thumbfile = feh_thumbnail_get_selected_file();
+
+			if (thumbfile) {
+				feh_action_run(thumbfile, opt.actions[action]);
+
+				if (!opt.hold_actions[action])
+					feh_thumbnail_mark_removed(thumbfile, 0);
+			}
+		}
 	}
 	return;
 }
@@ -561,6 +568,10 @@ void feh_event_handle_keypress(XEvent * ev)
 	else if (feh_is_kp(&keys.zoom_in, keysym, state)) {
 		winwid->old_zoom = winwid->zoom;
 		winwid->zoom = winwid->zoom * 1.25;
+
+		if (winwid->zoom > ZOOM_MAX)
+			winwid->zoom = ZOOM_MAX;
+
 		winwid->im_x = (winwid->w / 2) - (((winwid->w / 2) - winwid->im_x) /
 			winwid->old_zoom * winwid->zoom);
 		winwid->im_y = (winwid->h / 2) - (((winwid->h / 2) - winwid->im_y) /
@@ -571,6 +582,10 @@ void feh_event_handle_keypress(XEvent * ev)
 	else if (feh_is_kp(&keys.zoom_out, keysym, state)) {
 		winwid->old_zoom = winwid->zoom;
 		winwid->zoom = winwid->zoom * 0.80;
+
+		if (winwid->zoom < ZOOM_MIN)
+			winwid->zoom = ZOOM_MIN;
+
 		winwid->im_x = (winwid->w / 2) - (((winwid->w / 2) - winwid->im_x) /
 			winwid->old_zoom * winwid->zoom);
 		winwid->im_y = (winwid->h / 2) - (((winwid->h / 2) - winwid->im_y) /
